@@ -4,199 +4,20 @@
 # @Author : boliqq07
 # @Software: PyCharm
 # @License: MIT License
-from abc import abstractmethod
 
-import multiprocessing
-import numpy as np
 import os
+from typing import List
+
+import numpy as np
 import pandas as pd
-import pathlib
-import warnings
 from path import Path
 from pymatgen.electronic_structure.core import OrbitalType, Spin
 from pymatgen.electronic_structure.dos import CompleteDos
 from pymatgen.io.vasp import Poscar, Vasprun
 from scipy.integrate import simps
 from scipy.ndimage import gaussian_filter1d
-from tqdm import tqdm
-from typing import Union, Callable, List, Any
 
-
-class _BasePathOut:
-    """
-    The subclass should fill the follow properties and functions.
-
-    self.necessary_files = []  # fill
-    self.out_file = ""  # fill
-    self.software = []  # fill
-    self.inter_temp_files = []  # fill
-    self.key_help = []  # fill
-
-    def run(self, path: Path, files: List = None):
-        ...  # fill
-
-    def batch_after_treatment(self, paths, res_code):
-        ...  # fill
-
-    """
-
-    def __init__(self, n_jobs: int = 1, tq: bool = True, store_single=False, log=True):
-        self.n_jobs = n_jobs
-        self.tq = tq
-        self.log = log
-        self.store_single = store_single
-        self.necessary_files = []  # fill
-        self.out_file = ""  # fill
-        self.software = []  # fill
-        self.inter_temp_files = []  # fill
-        self.key_help = ""  # fill
-
-    def __str__(self):
-        return f"{self.__class__.__name__} <necessary file: {self.necessary_files}, out file: {self.out_file}>"
-
-    def __repr__(self):
-        return f"{self.__class__.__name__} <necessary file: {self.necessary_files}, out file: {self.out_file}>"
-
-    def _to_path(self, path: Union[os.PathLike, Path, pathlib.Path, str]) -> Path:
-        if isinstance(path, str):
-            path = path.replace("\n", "")
-        return Path(path).abspath() if not isinstance(path, Path) else path
-
-    def convert(self, path: Union[os.PathLike, Path, pathlib.Path, str]):
-        self.check_software()
-        path = self._to_path(path)
-        path_bool = self.check_path_and_file(path)
-        if path_bool:
-            return self.wrapper(path)
-        else:
-            raise FileNotFoundError
-
-    def transform(self, paths: List[Union[os.PathLike, Path, pathlib.Path, str]]):
-        self.check_software()
-        paths = [self._to_path(pi) for pi in paths]
-        paths_bool = [self.check_path_and_file(pi) for pi in paths]
-        if self.n_jobs == 1:
-            res_code = [self.wrapper(pi) if pib is True else None for pi, pib in zip(paths, paths_bool)]
-        else:
-            # trues_index = np.where(np.array(paths_bool))[0]
-            trues = [i for i, b in zip(paths, paths_bool) if b]
-            res_code = self.parallelize_imap(self.wrapper, iterable=list(trues), n_jobs=self.n_jobs, tq=self.tq)
-
-        return self.batch_after_treatment(paths, res_code)
-
-    @staticmethod
-    def parallelize_imap(func: Callable, iterable: List, n_jobs: int = 1, tq: bool = True):
-        """
-        Parallelize the function for iterable.
-        """
-        pool = multiprocessing.Pool(processes=n_jobs)
-        if tq:
-            result_list_tqdm = [result for result in tqdm(pool.imap(func=func, iterable=iterable),
-                                                          total=len(iterable))]
-            pool.close()
-        else:
-            result_list_tqdm = [result for result in pool.imap(func=func, iterable=iterable)]
-            pool.close()
-
-        return result_list_tqdm
-
-    def check_path_and_file(self, path: Path):
-        """1.Check the path exist, and necessary file."""
-        if not path.exists():
-            return False
-        else:
-            if path.isdir():
-                if all([True for i in self.necessary_files if (path / i).exists()]):
-                    return True
-                else:
-                    warnings.warn(f"Loss necessary files for {path}.")
-                    return False
-            elif path.isfile():
-                warnings.warn("If path is one file, we think the necessary_files is just this file, "
-                              "the ``necessary_files`` would be empty,"
-                              "and with no follow-up necessary file inspection.")
-                self.necessary_files = []
-                return True
-            else:
-                return False
-
-    def check_software(self):
-        """2.Check software exist (just for linux). """
-        for si in self.software:
-            res = os.popen(f"which {si}").readlines()
-            if len(res) == 0:
-                raise NotImplementedError(f"Can't find the {si}, please:\n"
-                                          f"1. Add this software to your PATH for system,\n"
-                                          f"   such as, add 'export PATH=$PATH:~/bin/chgsum.pl' to '~/.bashrc' ;\n"
-                                          f"2. keep access permission by such as, 'chmod u+x ~/bin/chgsum.pl' .")
-
-    def wrapper(self, path: Path):
-        """For exception."""
-        if path.isfile():
-            files = [path, ]  # 单个文件
-        else:
-            files = [path / i for i in self.necessary_files]
-
-        old = os.getcwd()
-        os.chdir(path)
-        try:
-            result = self.run(path, files)
-            if result is None:
-                print("No return for:", path)
-                if self.log:
-                    print(self.log_txt)
-            else:
-                print("Ok for:", path)
-            os.chdir(old)
-            return result
-        except BaseException as e:
-            print(e)
-            print("Error for:", path)
-            if self.log:
-                print(self.log_txt)
-            os.chdir(old)
-            return None
-
-    @property
-    def log_txt(self):
-        return f"""
-        ##### HELP START #######"
-        1.Check the necessary files, and software:
-        {self.necessary_files}, {self.software}
-        2.Check the necessary files from key parameter:
-        {self.key_help}
-        3.Check the intermediate temporary files are generated in path:
-        {self.inter_temp_files}
-        "##### HELP END #######"
-        """
-
-    @abstractmethod
-    def run(self, path: Path, files: List = None) -> Any:  # fill
-        """3.Run with software and necessary file and get data.
-        (1) Return result in code, or (2) Both return file to each path and in code."""
-
-        # 可以更简单的直接写命令，而不用此处的文件名 (files), 但是需要保证后续出现的文件、软件与 necessary_file, software 一致
-        # 函数强制返回除去None的对象，用于后续检查!!! (若返回None,则认为run转换错误)
-
-        # Sample
-        # vasprun = Vasprun(path/"vasprun.xml")
-        # ef = vasprun.efermi
-        # return {"efermi":ef}
-
-        return {"File": path}
-
-    @abstractmethod
-    def batch_after_treatment(self, paths, res_code):  # fill
-        """4. Organize batch of data in tabular form, return one or more csv file. (force!!!)."""
-        # 合并多个文件到一个csv.
-
-        # Sample
-        # data_all = {pi:ri for pi,ri in zip(paths,res_code)}
-        # result = pd.DataFrame(data_all).T
-        # result.to_csv(self.out_file)
-        # print("'{}' are sored in '{}'".format(self.out_file, os.getcwd()))
-
-        return pd.DataFrame.from_dict({"File": paths}).T
+from featurebox.cli._basepathout import _BasePathOut
 
 
 class Dosxyz:
@@ -207,16 +28,16 @@ class Dosxyz:
 
     number_of_header_lines = 6
 
-    def __init__(self, doscar="DOSCAR", poscar="POSCAR", vasprun="vasprun.xml",
+    def __init__(self, doscar="DOSCAR", poscar="CONTCAR", vasprun="vasprun.xml",
                  ispin=2, lmax=2, lorbit=11, spin_orbit_coupling=False, read_pdos=True, max=10, min=-10):
         """
         Create a Doscar object from a VASP DOSCAR file.
+
         Args:
             doscar (str): Filename of the VASP DOSCAR file to read.
             poscar (str): File POSCAR/CONTCAR
             vasprun (str): File vasprun.xml
-            ispin (optional:int): ISPIN flag. Set to 1 for non-spin-polarised or 2 for spin-polarised calculations.
-            Default = 2.
+            ispin (optional:int): ISPIN flag. Set to 1 for non-spin-polarised or 2 for spin-polarised calculations. (Default = 2.)
             lmax (optional:int): Maximum l angular momentum. (d=2, f=3). Default = 2.
             lorbit (optional:int): The VASP LORBIT flag. (Default=11).
             max (optional:int): max value
@@ -577,11 +398,11 @@ class Dosxyz:
 
 
 class DosxyzPathOut(_BasePathOut):
-    """Get d band center from paths and return csv file."""
+    """Get dos from paths and return csv file."""
 
     def __init__(self, n_jobs: int = 1, tq: bool = True, store_single=False, method="ele"):
         super(DosxyzPathOut, self).__init__(n_jobs=n_jobs, tq=tq, store_single=store_single)
-        self.necessary_files = ["vasprun.xml", "DOSCAR", "POSCAR"]
+        self.necessary_files = ["vasprun.xml", "DOSCAR", "CONTCAR"]
         self.out_file = "dos_xyz_py_all.csv"
         self.software = []
         self.method = method
@@ -592,7 +413,7 @@ class DosxyzPathOut(_BasePathOut):
         # 可以更简单的直接写命令，而不用此处的文件名 (files), 但是需要保证后续出现的文件、软件与 necessary_file, software 一致
         # 函数强制返回除去None的对象，用于后续检查!!! (若返回None,则认为run转换错误)
 
-        dos = Dosxyz(doscar="DOSCAR", poscar="POSCAR", vasprun="vasprun.xml", ispin=2, lmax=2,
+        dos = Dosxyz(doscar="DOSCAR", poscar="CONTCAR", vasprun="vasprun.xml", ispin=2, lmax=2,
                      lorbit=11, read_pdos=True, max=10, min=-10)
 
         if self.method == "ele":
@@ -697,18 +518,6 @@ def get_atom_pdos(dos: CompleteDos = None, mark_orbital=None, mark_atom_numbers=
     return spd_dos_res
 
 
-def get_atom_dos():
-    pass
-
-
-# def get_pdos_percent(spd_dos):
-#     dos_energy = spd_dos["energy"]
-#     total = spd_dos["total"]
-#     index = np.argmin(np.abs(np.array(dos_energy)))
-#     percent = {k: np.sum(v[index - 1:index + 1]) / np.sum(total[index - 1:index + 1]) for k, v in spd_dos.items()}
-#     return percent
-
-
 class DosPy(_BasePathOut):
     """Get d band center from paths and return csv file."""
 
@@ -755,31 +564,45 @@ class DosPy(_BasePathOut):
         return result
 
 
-class CLICommand:
+class _CLICommand:
     """
     批量提取 DOS，保存到当前工作文件夹。 查看参数帮助使用 -h。
 
-    在 featurebox 中运行，请使用 featurebox dos ...
-    若复制本脚本并单运行，请使用 python dos ...
-
     Notes:
+
         1.1 前期准备
         INCAR, CONTCAR, DOSCAR
 
+        1.2 INCAR准备
+        INCAR文件参数要求：
+        LORBIT=11
+        NSW = 0
+        IBRION = -1
+
         2.运行文件要求:
-        vaspkit <= 1.2.1, for -j in (0,1,2)
+        vaspkit <= 1.2.1, for -j in (0,1)
 
-    如果在 featurebox 中运行多个案例，请指定路径所在文件:
+    -j 参数说明：
 
-    Example:
+        0       1
+        [DosPy, DosxyzPathOut]
 
-    $ featurebox dos -f /home/sdfa/paths.temp
+        0: 调用pymatgen运行。
+        1: 调用此python代码运行。
 
-    如果在 featurebox 中运行单个案例，请指定运算子文件夹:
+    补充:
 
-    Example:
+        在 featurebox 中运行，请使用 featurebox dos ...
 
-    $ featurebox dos -p /home/parent_dir/***/sample_dir
+        若复制本脚本并单运行，请使用 python {this}.py ...
+
+        如果在 featurebox 中运行多个案例，请指定路径所在文件:
+
+        $ featurebox dos -f /home/sdfa/paths.temp
+
+        如果在 featurebox 中运行单个案例，请指定运算子文件夹:
+
+        $ featurebox dos -p /home/parent_dir/***/sample_dir
     """
 
     @staticmethod
@@ -788,15 +611,20 @@ class CLICommand:
         parser.add_argument('-f', '--paths_file', type=str, default='paths.temp')
         parser.add_argument('-j', '--job_type', type=int, default=0)
 
+    @staticmethod
+    def parse_args(parser):
+        return parser.parse_args()
+
     @staticmethod
     def run(args, parser):
         methods = [DosPy, DosxyzPathOut]
         pf = Path(args.paths_file)
         pn = Path(args.path_name)
         if pf.isfile():
-            bad = methods[args.job_type](n_jobs=1, store_single=True)
+            bad = methods[args.job_type](n_jobs=4, store_single=True)
             with open(pf) as f:
                 wd = f.readlines()
+            assert len(wd) > 0, f"No path in file {pf}"
             bad.transform(wd)
         elif pn.isdir():
             bad = methods[args.job_type](n_jobs=1, store_single=True)
@@ -808,29 +636,14 @@ class CLICommand:
 if __name__ == '__main__':
     """
     Example:
-
         $ python this.py -p /home/dir_name
+        $ python this.py -f /home/dir_name/path.temp
     """
     import argparse
 
-    parser = argparse.ArgumentParser(description="Get DOS. Example:\n"
-                                                 "python this.py -p /home/dir_name")
-    parser.add_argument('-p', '--path_name', type=str, default='.')
-    parser.add_argument('-f', '--paths_file', type=str, default='paths.temp')
-    parser.add_argument('-j', '--job_type', type=int, default=1)
-
-    args = parser.parse_args()
-    # run
-    methods = [DosPy, DosxyzPathOut]
-    pf = Path(args.paths_file)
-    pn = Path(args.path_name)
-    if pf.isfile():
-        bad = methods[args.job_type](n_jobs=1, store_single=True)
-        with open(pf) as f:
-            wd = f.readlines()
-        bad.transform(wd)
-    elif pn.isdir():
-        bad = methods[args.job_type](n_jobs=1, store_single=True)
-        bad.convert(pn)
-    else:
-        raise NotImplementedError("Please set -f or -p parameter.")
+    parser = argparse.ArgumentParser(description=f"Get data by {__file__}. Examples:\n"
+                                                 "python this.py -p /home/dir_name , or\n"
+                                                 "python this.py -f /home/dir_name/paths.temp")
+    _CLICommand.add_arguments(parser=parser)
+    args = _CLICommand.parse_args(parser=parser)
+    _CLICommand.run(args=args, parser=parser)
